package collapsecalendarviewwrapper;

import main.java.com.wefika.calendar.CollapseCalendarView;
import main.java.com.wefika.calendar.manager.CalendarManager;

import org.joda.time.LocalDate;

import android.content.Context;
import android.view.ViewGroup;
import anywheresoftware.b4a.BA;
import anywheresoftware.b4a.BA.Author;
import anywheresoftware.b4a.BA.Hide;
import anywheresoftware.b4a.BA.Pixel;
import anywheresoftware.b4a.BA.ShortName;
import anywheresoftware.b4a.BA.Version;
import anywheresoftware.b4a.BALayout;
import anywheresoftware.b4a.BA.DependsOn;
import anywheresoftware.b4a.keywords.Common.DesignerCustomView;
import anywheresoftware.b4a.objects.LabelWrapper;
import anywheresoftware.b4a.objects.PanelWrapper;
import anywheresoftware.b4a.objects.ViewWrapper;
import anywheresoftware.b4a.BA.ActivityObject;
import anywheresoftware.b4a.BA.Events;

import android.view.View;
import android.graphics.Color;

import android.os.Bundle;
import android.app.Activity;

//@ActivityObject
@Events(values={"date_selected(date As String)"})
@Author("Github: Blaz Solar, Wrapped by: Johan Schoeman")
@Version(1.00f)
@ShortName("CollapseCalendarView")
@DependsOn(values={"android-support-v4", "commons-lang3-3.4", "joda-time-2.0"})
public class collapseCalendarViewWrapper extends ViewWrapper<CollapseCalendarView> implements DesignerCustomView {


	private BA ba;
	private String eventName;
    private CollapseCalendarView cv;
	private int minusyears = 0;
	private int plusyears = 0;
	
	
	/**
     * Initialize with the name of the Event that will be raised when a date is clicked,
     * the number of years backwards from the present date (0, 1, 2, 3......), and
     * the number of years forward from the present date (0, 1, 2, 3......)	 
     */		
	public void Initialize(BA ba, String EventName, int minusyears, int plusyears) {
		this.minusyears = minusyears;
		this.plusyears = plusyears;
		_initialize(ba, null, EventName);
	}

	@Hide
	public void _initialize(BA ba, Object activityClass, String EventName) {
		this.eventName = EventName.toLowerCase(BA.cul);
		this.ba = ba;
		cv = new CollapseCalendarView(ba.context);
        this.setObject(cv);
		
       addListener();
       init1();
 	}
	
	private void init1() {
        CalendarManager manager = new CalendarManager(LocalDate.now(), CalendarManager.State.MONTH, LocalDate.now().minusYears(minusyears), LocalDate.now().plusYears(plusyears));
        cv.init(manager);		
	}
	
	
    private void addListener() {
 
		cv.setListener(new CollapseCalendarView.OnDateSelect() {
			@Override
			public void onDateSelected(LocalDate date) {
				
              if (ba.subExists(eventName + "_date_selected")) {
				ba.raiseEventFromDifferentThread(getObject(), null, 0, eventName + "_date_selected", true, new Object[]{date.toString()});
              }				
			}	
		});
	}	
	
	
	//programmatically add view
    @Hide
	public void AddToParent(ViewGroup Parent, @Pixel int left, @Pixel int top, @Pixel int width, @Pixel int height) {
		Parent.addView(cv, new BALayout.LayoutParams(left, top, width, height));
	}
	
	//this method cannot be hidden.
  
        
	public void DesignerCreateView(PanelWrapper base, LabelWrapper lw, anywheresoftware.b4a.objects.collections.Map props) {
		ViewGroup vg = (ViewGroup) base.getObject().getParent();
		AddToParent(vg, base.getLeft(), base.getTop(), base.getWidth(), base.getHeight());
		base.RemoveView();
		//set text properties
	}



	public void setLeft(int left)	{
		BALayout.LayoutParams lp = (BALayout.LayoutParams)cv.getLayoutParams();
		lp.left = left;
		cv.getParent().requestLayout();
	}

	public int getLeft()	{
		BALayout.LayoutParams lp = (BALayout.LayoutParams)cv.getLayoutParams();
		return lp.left;
	}
	public void setTop(int top)	{
		BALayout.LayoutParams lp = (BALayout.LayoutParams)cv.getLayoutParams();
		lp.top = top;
		cv.getParent().requestLayout();
	}
 
	public int getTop()	{
		BALayout.LayoutParams lp = (BALayout.LayoutParams)cv.getLayoutParams();
		return lp.top;
	}
	
	public void setWidth(int Width)	{
		BALayout.LayoutParams lp = (BALayout.LayoutParams)cv.getLayoutParams();
		lp.width = Width;
		cv.getParent().requestLayout();
	}

	public int getWidth()	{
		BALayout.LayoutParams lp = (BALayout.LayoutParams)cv.getLayoutParams();
		return lp.width;
	}
	
	public void setHeight(int height)	{
		BALayout.LayoutParams lp = (BALayout.LayoutParams)cv.getLayoutParams();
		lp.height = height;
		cv.getParent().requestLayout();
	}

	public int getHeight()	{
		BALayout.LayoutParams lp = (BALayout.LayoutParams)cv.getLayoutParams();
		return lp.height;
	}
	

	/**
     * Returns the date that has been clicked on
     */		
    public String getSelectedDate() {
	    return cv.getSelectedDate().toString();
	}


	/**
     * Returns the minimum date that the calendar has been set to
     */		
    public String getMinDate() {
        return cv.getMinDate().toString();
    }
	
	
	/**
     * Set the number of years backwards from the present date
	 * This method should be used with "plusYears = x" where x = 0, 1, 2, 3, .....
	 * The range that the calendar will display will be from (present date - minusyears) to (present date + plusyears)
     */			
     public void setminusYears(int minusyears) {	
	    this.minusyears = minusyears;
        CalendarManager manager = new CalendarManager(LocalDate.now(), CalendarManager.State.MONTH, LocalDate.now().minusYears(minusyears), LocalDate.now().plusYears(plusyears));
        cv.init(manager);				
		
    }
	
	/**
     * Set the number of years forward from the present date
	 * This method should be used with "minusYears = x" where x = 0, 1, 2, 3, .....
	 * The range that the calendar will display will be from (present date - minusyears) to (present date + plusyears)
     */			
    public void setplusYears(int plusyears) {	
	    this.plusyears = plusyears;
        CalendarManager manager = new CalendarManager(LocalDate.now(), CalendarManager.State.MONTH, LocalDate.now().minusYears(minusyears), LocalDate.now().plusYears(plusyears));
        cv.init(manager);				
    }	


	/**
     * Returns the maximum date that the calendar has been set to
     */		
    public String getMaxDate() {
        return cv.getMaxDate().toString();
    }
	

	private int minusmonths = 0;
	/**
     * Set the number of months backwards from the present date
	 * This method should be used with "plusMonths = x" where x = 0, 1, 2, 3, .....
	 * The range that the calendar will display will be from (present date - minusmonths) to (present date + plusmonths)
     */			
    public void setminusMonths(int minusmonths) {	
	    this.minusmonths = minusmonths;
        CalendarManager manager = new CalendarManager(LocalDate.now(), CalendarManager.State.MONTH, LocalDate.now().minusMonths(minusmonths), LocalDate.now().plusMonths(plusmonths));
        cv.init(manager);				
		
    }
	
	private int plusmonths = 0;
	/**
     * Set the number of months forward from the present date
	 * This method should be used with "minusMonths = x" where x = 0, 1, 2, 3, .....
	 * The range that the calendar will display will be from (present date - minusmonths) to (present date + plusmonths)
     */		
    public void setplusMonths(int plusmonths) {	
	    this.plusmonths = plusmonths;
        CalendarManager manager = new CalendarManager(LocalDate.now(), CalendarManager.State.MONTH, LocalDate.now().minusMonths(minusmonths), LocalDate.now().plusMonths(plusmonths));
        cv.init(manager);				
    }		



	/**
     * Set the number of months back and the number of years forward from the present date
	 * This is the range that the calendar will display
     */	
    public void setminusMonthsPlusYears(int minusmonths, int plusyears) {	
	    this.minusmonths = minusmonths;
		this.plusyears = plusyears;
        CalendarManager manager = new CalendarManager(LocalDate.now(), CalendarManager.State.MONTH, LocalDate.now().minusMonths(minusmonths), LocalDate.now().plusYears(plusyears));
        cv.init(manager);				
		
    }
	

	/**
     * Set the number of years back and the number of months forward from the present date
	 * This is the range that the calendar will display	 
     */	
    public void setminusYearsPlusMonths(int minusyears, int plusmonths) {	
	    this.plusmonths = plusmonths;
		this.minusyears = minusyears;
        CalendarManager manager = new CalendarManager(LocalDate.now(), CalendarManager.State.MONTH, LocalDate.now().minusYears(minusyears), LocalDate.now().plusMonths(plusmonths));
        cv.init(manager);				
    }		

	
	/**
     * Set the color of the Day names
     */		
    public void setDayNameColor(int color) {
		cv.setDayNameColor(color);
		cv.invalidate();
	}	
	
	/**
     * Set the size of the Day names
     */	
    public void setDayNameTextSize(int daynametextsize) {
		cv.setDayNameTextSize(daynametextsize);
		cv.invalidate();
	}	

	/**
     * Set the color of the Title Text (i.e the Month/Year and the Week Number)
     */
    public void setTitleTextColor(int titletextcolor) {
		cv.setTitleTextColor(titletextcolor);
		cv.invalidate();
	}

	
	/**
     * Set the size of the Title Text (i.e the Month/Year and the Week Number)
     */
    public void setTitleTextSize(int titletextsize) {
		cv.setTitleTextSize(titletextsize);
		cv.invalidate();
	}		
   
}
